<?php

namespace Spec\Infomaker\Everyware\Base\OpenContent;

use Infomaker\Everyware\Base\OpenContent\QueryBuilder;
use Infomaker\Everyware\Support\Date;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class QueryBuilderSpec extends ObjectBehavior
{
    function it_is_initializable()
    {
        $this->beConstructedWith('');
        $this->shouldHaveType(QueryBuilder::class);
    }

    public function it_should_set_the_query()
    {
        $this->beConstructedWith('query');

        $this->buildQueryString()->shouldReturn('query');
    }

    public function it_should_seperate_query_and_text_search()
    {
        $this->beConstructedWith('query:true', 'text search');

        $this->buildQueryString()->shouldReturn('("text search") AND query:true');
    }

    public function it_can_append_queries()
    {
        $this->beConstructedWith('contenttype:Article');

        $this->append('published:true')->append('new:"Query string"');

        $this->buildQueryString()->shouldReturn(
            'contenttype:Article' .
            ' AND published:true' .
            ' AND new:"Query string"'
        );
    }

    public function it_can_prepend_queries()
    {
        $this->beConstructedWith('contenttype:Article');

        $this->prepend('published:true')
            ->prepend('new:"Query string"', 'OR');

        $this->buildQueryString()->shouldReturn(
            'new:"Query string"' .
            ' OR published:true' .
            ' AND contenttype:Article'
        );
    }

    public function it_can_append_queries_with_helpers()
    {
        $this->beConstructedWith('');

        $this->append('contenttype:Article')
            ->andIf('published:true')
            ->orIf('other:"Query string"')
            ->andIfNot('new:"Query string"')
            ->orIfNot('else:"Query string"');

        $this->buildQueryString()->shouldReturn(
            'contenttype:Article' .
            ' AND published:true' .
            ' OR other:"Query string"' .
            ' AND NOT new:"Query string"' .
            ' OR NOT else:"Query string"'
        );
    }

    public function it_can_append_queries_by_searching()
    {
        $this->beConstructedWith('');

        $this->append('contenttype:Article')
            ->andIfProperty('published', true)
            ->orIfProperty('other', 'Query string')
            ->andIfNotProperty('new', 'Query string')
            ->orIfNotProperty('else', 'Query string');

        $this->buildQueryString()->shouldReturn(
            'contenttype:Article' .
            ' AND published:true' .
            ' OR other:"Query string"' .
            ' AND NOT new:"Query string"' .
            ' OR NOT else:"Query string"'
        );
    }

    public function it_can_determine_if_a_query_is_empty()
    {
        $this->beConstructedWith('');

        $this->isEmpty()->shouldReturn(true);
        $this->append('published', true);
        $this->isEmpty()->shouldReturn(false);
    }

    public function it_has_helper_functions_for_creating_property_query()
    {
        $this->beConstructedWith('');
        self::createProperty('contenttype','Article')->shouldReturn('contenttype:Article');
        self::createProperty('published',true)->shouldReturn('published:true');
        self::createProperty('new', 'Query string')->shouldReturn('new:"Query string"');
    }

    public function it_has_helper_functions_for_creating_property_search()
    {
        $this->beConstructedWith('');
        self::propertySearch('contenttype',['Article', 'Concept'])->shouldReturn('(contenttype:Article OR contenttype:Concept)');
        self::propertySearch('contenttype',['Article', 'Concept'], 'AND')->shouldReturn('(contenttype:Article AND contenttype:Concept)');
    }

    public function it_has_helper_functions_for_creating_date_property_search()
    {
        $this->beConstructedWith('');
        self::dateSpanSearch('Pubdate')->shouldReturn('Pubdate:[* TO NOW]');
        self::dateSpanSearch('Pubdate','2018-10-04', '2018-10-06')->shouldReturn('Pubdate:[2018-10-03T22:00:00Z TO 2018-10-05T22:00:00Z]');
    }

    public function it_has_helper_functions_for_creating_date_property_search_using_date_class()
    {
        $from_date = Date::parse('2018-10-06');
        $this->beConstructedWith('');
        self::dateSpanSearch('Pubdate', $from_date->copy()->subDays(2), $from_date)->shouldReturn('Pubdate:[2018-10-03T22:00:00Z TO 2018-10-05T22:00:00Z]');
    }

    public function it_can_create_a_date_string_usable_by_open_content()
    {
        $this->beConstructedWith('');
        $from_date = Date::parse('2018-10-06');
        self::createOcDate('2018-10-06')->shouldReturn('2018-10-05T22:00:00Z');
    }
}
